plot.mdt <-
function(mdt, nodes = "all", ages = "SD", bars = "SD", nodes.mdt = FALSE, digits = 0, barcol = "blue", barwd = 2, agecex = 0.7, agefont = 2,agecol = "black",  agebg = "#FF4000FF", ageframe = "r", ageadj = 0.5, label = " Mya", points = TRUE, x.lim = FALSE, cex = 1, edge.color = "black", edge.lty = 1, tip.color = "black", ...){
	
	if (class(mdt) != "mdt") 							stop("mdt object is not of class mdt")
	
	# determine the x.lim values 
	# _________________________________________________________
	b <- mdt$TABLE[dim(mdt$TABLE)[1],2]
	a <- 0.015 * b * max(nchar(mdt$TREE$tip.label)) * cex
	if (bars == "CI" || bars == "SD"){
		if (bars == "CI")
			c <- mdt$TABLE[dim(mdt$TABLE)[1],5]
		if (bars == "SD")
			c <- b + mdt$TABLE[dim(mdt$TABLE)[1],3]
	}												else c <- b
		
	lo <- (a + b) / 100
	
	if (x.lim[1]){
		if (!is.numeric(x.lim))
			x <- c(b - c, a + b + lo)
		else {
			if (length(x.lim) == 1)
				x <- c(b - c, x.lim)
			else 
				x <- x.lim	
		}
	}												else
		x <- c(0, a + b + lo)
		
		
	# dummy plot 
	# _________________________________________________________			
	plot.phylo(mdt[[1]], edge.color = "white", tip.color = 		"white", x.lim = x, label.offset = lo, cex = cex, edge.lty = edge.lty, ...)
	r <- get("last_plot.phylo", envir = .PlotPhyloEnv)
	# plots time scale
	if (is.character(label) || is.na(label)){
		axisPhylo()
	# Label the time scale according to the labels.offset of 
	# the tiplabels
	# __________________________________________________________
		maxi <- max(r$xx)
		mtext(label, side = 1, line = 1, 					at = c(maxi + r$label.offset), adj = 0, font = 0)
	}
	# get necessary values
	t <- mdt[[2]] # table with estimates
	L <- length(mdt[[1]]$tip.label) # number of tips
	N <- mdt[[1]]$Nnode # number of internal nodes
	#node.mdt <- t[,1] # mdt node ordering
    cons.mdt <- as.integer(mdt[[3]][2,]) # mdt nodes of constraints
    cons.ape <- t[cons.mdt + 1, 6] # same according to ape
 
 
    if (nodes[1] == "all" || nodes[1] == "allButCons"){
    	t <- t[order(t[,6]),]
    	# create node vector for nodes = "all"
    	if (nodes[1] == "all")	
        	nodes <- (L+1):(L + N)
        # create node vector for nodes = "allButCons"
    	if (nodes[1] == "allButCons"){
    		nodes <- (L+1):(L + N)
    		nodes <- nodes[-which(nodes %in% cons.ape)]
		}    
    }	
    
    if (!nodes.mdt)
    	t <- t[order(t[,6]),]						else 
    	t <- t[-1,]	

    		     	
	# calculate values to plot
	# __________________________________________________________
   	XX <- r$xx[nodes]
  	YY <- r$yy[nodes]
    age <- t[nodes, 2]
    age <- round(age, digits = digits)
    sd <- t[nodes, 3]
    sd <- round(sd, digits = digits)
    min <- t[nodes, 4]
    max <- t[nodes, 5]
    MAX <- max(t[, 2])
    min <- MAX-min
    max <- MAX-max
    
    # calculate coordintes of error bars and plot error bars
    # __________________________________________________________
   	if (bars == "SD"){	
    	x1 <- XX - sd
		x2 <- XX + sd
		segments(x1, YY, x2, YY, lwd = barwd, col = barcol)
	}  
	if (bars == "CI") {	
		x1 <- min
		x2 <- max
		segments(x1, YY, x2, YY, lwd = barwd, col = barcol)
	} 
		
   	# plot chronogram
   	# ---------------
    plot.phylo.upon(mdt[[1]], edge.color = edge.color, 		tip.color = tip.color, x.lim = x,label.offset = lo, 		cex = cex, edge.lty = edge.lty, ...)
    #   plot a little dot on the nodes of the bars
	    if (points) 
    	nodelabels(text = rep(NA, length(nodes)), nodes, 			frame = "c", bg = "white", cex = 1)
    # plot ages (and S.D.)
    if (ages == "SD"){
    		for (i in seq(along = age))
    			nodelabels(text = substitute(x %+-% y, 					list(x = age[i], y = sd[i])), node = nodes[i], 					frame = ageframe, adj = ageadj, cex = agecex, 					col = agecol, bg = agebg)   
    }
    if (ages == "mean"){
       	nodelabels(text = age, node = nodes, frame = ageframe, 			adj = ageadj, cex = agecex, col = agecol, bg = agebg, 			font = agefont)
	}
	
}   