\name{multidivtime}
\alias{multidivtime}
\title{Bayesian Relaxed Clock Molecular Dating}

\description{
This function performs Bayesian relaxed clock molecular dating as described by Jeffrey Thorne and Hirohisa Kishino.  It connects their original C code programs(paml2modelinf, estbranches, multidivtime) as well as Ziheng Yangs baseml program via R code in order to make the analysis more user friendly.  Additional functionality offered by the R code is automatic handling of age constraints, calculating the median of branch lengths and graphical output. 
}

\usage{
multidivtime(x, file = NULL, start = "baseml", part = 1,
	runs = 1, path = NULL, transfer.files = TRUE, 
	LogLCheck = 100, plot = TRUE)
}

\arguments{
	
  \item{x}{An object of class \code{"mdt.in"}, see \code{\link{mdt.in}}.}
  
  \item{start}{A vector indicating wether to start the analysis with the baseml, the estbranches, or the multidivtime step.}
  
  \item{part}{An integer indicating the partition to start with.}
  
  \item{runs}{An integer indicating how many MCMCs to run.}
  
  \item{file}{A vector giving the name for the output; default is \code{LAGOUT}.}
  
  \item{path}{A vector giving the path to the executables.}
  
  \item{transfer.files}{A logical indicating whether output files should be stored in a newly created folder in the working directory.}
  
  \item{LogLCheck}{An integer giving the difference in log likelihood units in order to compare the performance of \code{baseml} and code{estbranches}.}
  
  \item{plot}{A logical indicating whether node assignment and position of constraints should be plotted on the screen.}
}

\details{
In order to use the \code{multidivtime} function, there must be a directory somewhere on your system which contains the executables of \code{baseml}, \code{paml2modelinf}, \code{estbranches}, and \code{multidivtime}, as well as a file called \code{inseed} containing a number which is used to create random seeds for the Markov chain Monte Carlo runs.  Please note that all these files must be placed in a single directory, as \code{path} takes only one single argument.  The default for \code{path} is \code{pathMDT}, which means the user could save the path as an object called \code{pathMDT} and load it into the work space prior to execution of \code{multidivtime}.  Otherwise the user would have to specify the \code{path} argument each time explicitly.

The programs \code{baseml} and the \code{multidivtime} receive their arguments via two control files named \code{\link{baseml.ctr}} and \code{\link{multicntrl.dat}}.  Both objects have to be loaded into the work space prior to execution of \code{multidivtime}. 

}

\value{
  list of class \code{"mdt"} containing the following components:
   \item{TREE}{object of class "phylo", i.e. the chronogram.}
   \item{TABLE}{data frame containing estimates for mean node age, standard deviation of node age, and maximum and minimum value of the 95 \% confidence interval. The first column refers to the node numbering as used by \code{multidivtime}; the last column gives their corresponding translation to \code{ape} node numbering.}
   \item{CONSTRAINTS}{data frame containing  information about the constraints used for analysis.}
}

\references{
Kishino, H., J.L. Thorne, and W.J. Bruno. 2001. Performance of a divergence time estimation method under a probabilistic model of rate evolution. Mol. Biol. Evol. 18: 352-361.

Thorne, J.L., H. Kishino, and I.S. Painter. 1998. Estimating the rate of evolution of the rate of molecular evolution. Mol. Biol. Evol. 15: 1647-1657.

Thorne, J.L. and H. Kishino. 2002. Divergence time and evolutionary rate estimation with multilocus data. Syst. Biol. 51: 689-702.

Yang, Z. 2007. PAML 4: a program package for phylogenetic analysis by maximum likelihood.  Molecular Biology and Evolution 24: 1586-1591. \url{http://abacus.gene.ucl.ac.uk/software/paml.html}
}

\author{Christoph Heibl and Natalie Cusimano}

\seealso{\code{\link{mdt.in}},
\code{\link{mdt.nodes}},
\code{\link{read.mdt}},
\code{\link{plot.mdt}},
\code{\link{plot.cons}}
}

\examples{
## Load test data.	
data(test)

## Create object of class 'mdt.in'.
x <- mdt.in(gene1, tree1, cons1)

## Load defaults for baseml and multidivtime control files.
data(baseml.ctr)
data(multicntrl.dat)

## Have a look at '?baseml.ctr' and 'multicntrl.dat' for how
## to change the defaults.

## Not run:
## Specify a path to the executables.
#path <- "/Applications/multidistribute"

## Start multidivtime on the data stored in x.
#multidivtime(x, path = path)

## Run another MCMC on the same data set, but with
## different constraints.
#x2 <- mdt.in(gene1, tree1, cons2)
#multidivtime(x2, path = path, start = "multidivtime")

## Run another three MCMC runs on data set x. 
#multidivtime(x, path = path, start = "multidivtime", runs = 3)
## End(Not run)
}