\name{plot.cons}
\alias{plot.cons}
\title{Plotting labels on constrained nodes}
\description{
This function plots colored boxes on the constrained nodes with or without the given age constraints or a user-defined label.
}
\usage{
plot.cons(mdt, bg = c("yellow", "pink"), label = "cons.age", ...)
}
\arguments{
  \item{mdt}{object of class mdt.}
  \item{bg}{vector of mode \code{"character"} and \code{length = 2} giving the background colors of the labels; the first element sets the background color for \bold{maximum} age constraints, the second element sets the background color for \bold{minimum} age constraints.}
  \item{label}{character string giving the text to be plotted onto the labels, defaults to the age constraints as used in \code{\link{multidivtime}}.}
  \item{\dots}{Further arguments passed to \code{nodelabels(ape)} function.}
}
\details{
}
\value{
 Plots age and/or bars to the constrained nodes. No values are returned.
}

\author{Natalie Cusimano (\email{cusimano@bio.lmu.de})}

\seealso{\code{\link{nodelabels}}, \code{\link{plot.mdt}}}

\examples{
	
# Load example object of class 'mdt'.
data(mdt.example)

# Plot chronogram with bars representing 95-percent confidence
# intervals ...
plot.mdt(mdt.example, ages = FALSE, bars = "CI", barwd = 4, 
	barcol = "green")
	
# ... and then add constraints in a default manner.
plot.cons(mdt.example)	

# If you want to plot time estimates on the nodes, you can choose
# to omit values of those labels that have been constrained with
# the 'allButCons' option ...
plot.mdt(mdt.example, nodes = "allButCons", ages = "mean", 
	ageframe = "c", agebg = "black", agecol = "white", 
	bars = "SD", barwd = 3)
	
# ... and then add constraints these nodes.
plot.cons(mdt.example, bg = c("yellow", "orange"), frame = "circ", 
	cex = 0.8)	
}
